from setuptools import setup, find_packages


def remove_duplicate_requirements(requirements):
    return list(set(requirements))


with open("README.md", "r") as f:
    long_description = f.read()

base_requirements = [
    "openai>=0.27.2",
    "tenacity==8.2.2",
    "rich==13.3.3",
    "prompt_toolkit==3.0.38",
    # 添加你的其他基本依赖
]

# 更新其他依赖的版本和列表，如果需要的话

anthropic_requirements = ["anthropic>=0.2.8"]
cohere_requirements = ["cohere>=4.3.1"]
hf_requirements = ["transformers>=4.27.4"]
bard_requirements = ["bardapi==0.1.11"]
langchain_requirements = ["langchain>=0.0.135"]
gradio_requirements = ["gradio>=3.34.0"]
pettingzoo_requirements = ["pettingzoo[classic]>=1.23.1", "chess==1.9.4"]
umshini_requirements = ["pettingzoo>=1.23.1", "pygame-ce>=2.2.1", "langchain>=0.0.135"]


all_backends = anthropic_requirements + cohere_requirements + hf_requirements + bard_requirements + \
               langchain_requirements
all_envs = remove_duplicate_requirements(pettingzoo_requirements + umshini_requirements)
all_requirements = all_backends + all_envs + gradio_requirements


pkg_name = "MAgIC_LLM"  # 更新为新的包名
pkg_version = "0.7.0"  # 更新版本号

print("Package Name:", pkg_name)
print("Version:", pkg_version)


setup(
    name=pkg_name,  # 更新为新的包名
    version=pkg_version,  # 更新版本号
    author="Zhiyuan Hu",  # 更新为你的名字
    author_email="zhiyuan.hu.bj@gmail.com",  # 更新为你的邮箱
    description="A new version of the chat arena with llm-agent",  # 添加一个描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zhiyuanhubj/llm-agent",  # 更新为新的 GitHub 仓库 URL
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    python_requires=">=3.7",
    install_requires=base_requirements,
    extras_require={
        # 如果有需要，更新额外的依赖
        "all_backends": all_backends,
        "all": all_requirements,
    },
)


